// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package v1

import (
	"net/http"
	"xingjia-management-gateway/apis"
	"xingjia-management-gateway/errors"
	"xingjia-management-gateway/impl/v1/contact"
	param_v1 "xingjia-management-gateway/param/v1"
	"xingjia-management-gateway/utils"

	"git.getensh.com/common/gopkgs/logger"
	"git.getensh.com/common/gopkgs/tasker/httptasker"
	"git.getensh.com/common/gopkgs/util"
	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
)

//
// @Summary 添加联系我们
// @Description 添加联系我们
// @Tags 联系我们
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.ContactAddBody true " "
// @Success 200 {object} v1.ContactAddResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/jt/contact [post]
func (c *Controller) JtContactAdd(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.ContactAddRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.ContactAddBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		if err != nil {
			return err
		}
		// 响应数据
		resp := param_v1.ContactAddResponse{}
		rpcReq := &apis.ContactAddRequest{
			SelfId:   tokenInfo.Uid,
			SelfName: tokenInfo.User,
			Addr:     req.Addr,
			Phone:    req.Phone,
			Email:    req.Email,
		}

		_, err = contact.ContactAdd(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "ContactAdd"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 修改联系我们
// @Description 修改联系我们
// @Tags 联系我们
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.ContactUpdateBody true " "
// @Success 200 {object} v1.ContactUpdateResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/jt/contact [put]
func (c *Controller) JtContactUpdate(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.ContactUpdateRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.ContactUpdateBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		if err != nil {
			return err
		}
		// 响应数据
		resp := param_v1.ContactUpdateResponse{}
		rpcReq := &apis.ContactAddRequest{
			SelfId:   tokenInfo.Uid,
			SelfName: tokenInfo.User,
			Id:       req.Id,
			Addr:     req.Addr,
			Phone:    req.Phone,
			Email:    req.Email,
		}

		_, err = contact.ContactAdd(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "ContactUpdate"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 删除联系我们
// @Description 删除联系我们
// @Tags 联系我们
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param id query int true " 记录id"
// @Success 200 {object} v1.ContactDelResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/jt/contact [delete]
func (c *Controller) JtContactDel(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.ContactDelRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.ContactDelQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		if err != nil {
			return err
		}
		// 响应数据
		resp := param_v1.ContactDelResponse{}
		rpcReq := &apis.ContactDelRequest{
			SelfId:   tokenInfo.Uid,
			SelfName: tokenInfo.User,
			Id:       req.Id,
		}

		_, err = contact.ContactDel(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "ContactDel"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 联系我们信息
// @Description 联系我们信息
// @Tags 联系我们
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Success 200 {object} v1.ContactInfoResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/jt/contact [get]
func (c *Controller) JtContactInfo(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.ContactInfoRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := param_v1.ContactInfoResponse{}
		rpcReq := &apis.ContactInfoRequest{}

		rpcRsp, err := contact.ContactInfo(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "ContactInfo"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		resp.Data = *rpcRsp

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}
