// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package v1

import (
	"net/http"
	"xingjia-management-gateway/apis"
	"xingjia-management-gateway/consts"
	"xingjia-management-gateway/errors"
	"xingjia-management-gateway/impl/v1/jt"
	param_v1 "xingjia-management-gateway/param/v1"
	"xingjia-management-gateway/utils"

	"git.getensh.com/common/gopkgs/logger"
	"git.getensh.com/common/gopkgs/tasker/httptasker"
	"git.getensh.com/common/gopkgs/util"
	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
)

//
// @Summary 添加党风廉洁
// @Description 添加党风廉洁
// @Tags 党风廉洁
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.JtContentAddBody true " "
// @Success 200 {object} v1.JtContentAddResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/jt/df [post]
func (c *Controller) JtDfAdd(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.JtContentAddRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.JtContentAddBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		if err != nil {
			return err
		}
		// 响应数据
		resp := param_v1.JtContentAddResponse{}
		rpcReq := &apis.JtContentAddRequest{
			FirstPics:     req.FirstPics,
			ContentType:   consts.OperationModuleDangfeng,
			Content:       req.Content,
			SelfId:        tokenInfo.Uid,
			SelfName:      tokenInfo.User,
			PublishStatus: req.PublishStatus,
			Title:         req.Title,
		}

		_, err = jt.JtContentAdd(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "jt.JtContentAdd"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 修改党风廉洁
// @Description 修改党风廉洁
// @Tags 党风廉洁
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.JtContentUpdateBody true " "
// @Success 200 {object} v1.JtContentUpdateResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/jt/df [put]
func (c *Controller) JtDfUpdate(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.JtContentUpdateRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.JtContentUpdateBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		if err != nil {
			return err
		}
		// 响应数据
		resp := param_v1.JtContentUpdateResponse{}
		rpcReq := &apis.JtContentUpdateRequest{
			FirstPics:     req.FirstPics,
			ContentType:   consts.OperationModuleDangfeng,
			Content:       req.Content,
			SelfId:        tokenInfo.Uid,
			SelfName:      tokenInfo.User,
			Id:            req.Id,
			PublishStatus: req.PublishStatus,
		}

		_, err = jt.JtContentUpdate(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "jt.JtContentUpdate"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 发布或下架党风廉洁
// @Description 发布或下架党风廉洁
// @Tags 党风廉洁
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.JtContentPublishBody true " "
// @Success 200 {object} v1.JtContentPublishResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/jt/df/publish [put]
func (c *Controller) JtDfPublish(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.JtContentPublishRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.JtContentPublishBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		if err != nil {
			return err
		}
		// 响应数据
		resp := param_v1.JtContentPublishResponse{}
		rpcReq := &apis.JtContentPublishRequest{
			ContentType:   consts.OperationModuleDangfeng,
			SelfId:        tokenInfo.Uid,
			SelfName:      tokenInfo.User,
			Id:            req.Id,
			PublishStatus: req.PublishStatus,
		}

		_, err = jt.JtContentPublish(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "jt.JtContentPublish"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 删除党风廉洁
// @Description 删除党风廉洁
// @Tags 党风廉洁
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param id query int true " 记录id"
// @Success 200 {object} v1.JtContentDelResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/jt/df [delete]
func (c *Controller) JtDfDel(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.JtContentDelRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.JtContentDelQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		if err != nil {
			return err
		}
		// 响应数据
		resp := param_v1.JtContentDelResponse{}
		rpcReq := &apis.JtContentDelRequest{
			ContentType: consts.OperationModuleDangfeng,
			SelfId:      tokenInfo.Uid,
			SelfName:    tokenInfo.User,
			Id:          req.Id,
		}

		_, err = jt.JtContentDel(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "jt.JtContentDel"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 党风廉洁列表
// @Description 党风廉洁列表
// @Tags 党风廉洁
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param page query int false " "
// @Param page_size query int false " "
// @Success 200 {object} v1.JtContentListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/jt/df [get]
func (c *Controller) JtDfList(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.JtContentListRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.JtContentListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetJwtTokenFromCtx(ctx)
		if err != nil {
			return err
		}
		// 响应数据
		resp := param_v1.JtContentListResponse{}
		rpcReq := &apis.JtContentListRequest{
			ContentType: consts.OperationModuleDangfeng,
			PageSize:    req.PageSize,
			Page:        req.Page,
			SelfId:      tokenInfo.Uid,
		}

		rpcRsp, err := jt.JtContentList(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "jt.JtContentList"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		if rpcRsp.List == nil {
			rpcRsp.List = make([]*apis.JtContentItem, 0)
		}
		resp.Data = *rpcRsp

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}
