// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package jt

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"strings"
	"time"
	"xingjia-management-gateway/apis"
	"xingjia-management-gateway/consts"
	"xingjia-management-gateway/errors"
	"xingjia-management-gateway/impl/v1/local_objs"
	"xingjia-management-gateway/impl/v1/operation_log"
	dbmodel "xingjia-management-gateway/model"
)

func checkJtContentAddParam(req *apis.JtContentAddRequest) error {
	switch {
	case consts.JtContentTable[req.ContentType] == "":
		return status.Error(10003, "未知内容类型")
	case req.SelfId == 0:
		return status.Error(10003, "未知操作者")
	case req.PublishStatus != 1 && req.PublishStatus != 2:
		return status.Error(10003, "请选择是否发布")
	}
	return nil
}

func getJtContent(p dbmodel.TJtContent) string {
	publishStatus := "发布"
	if p.PublishStatus != 1 {
		publishStatus = "暂不发布"
	}
	m := map[string]interface{}{
		"标题":   p.Title,
		"内容":   p.Content,
		"发布状态": publishStatus,
	}
	bytes, _ := json.Marshal(m)
	return string(bytes)
}

//
func JtContentAdd(ctx context.Context, req *apis.JtContentAddRequest) (reply *apis.JtContentAddReply, err error) {

	reply = &apis.JtContentAddReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	if err = checkJtContentAddParam(req); err != nil {
		return nil, err
	}
	now := time.Now()
	p := dbmodel.NewJtContent(consts.JtContentTable[req.ContentType])
	p.Content = req.Content
	p.PublishStatus = req.PublishStatus
	p.FirstPics = strings.Join(req.FirstPics, ";")
	p.CreatedAt = now
	p.UpdatedAt = now
	p.Title = req.Title

	db := database.DB().Begin()
	err = p.Insert(db)
	defer func() {
		if err != nil {
			db.Rollback()
		}
	}()
	if err != nil {
		return nil, errors.DataBaseError
	}

	// 操作日志
	content := getJtContent(*p)
	logReq := &apis.OperationLogAddRequest{
		OriginContent: "",
		TargetContent: content,
		Uid:           req.SelfId,
		User:          req.SelfName,
		Module:        int32(req.ContentType),
		OperationTime: now.Unix(),
		Operation:     consts.OperationActionAdd,
	}
	err = operation_log.OperationLogAddHandle(logReq, db)
	if err != nil {
		return nil, err
	}

	inList := req.FirstPics
	array := local_objs.GetObjFromText(req.Content)
	inList = append(inList, array...)

	err = local_objs.InAndOut(db, inList, []string{})
	if err != nil {
		return nil, err
	}

	db.Commit()

	return reply, nil
}
