// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package user

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"git.getensh.com/common/gopkgs/util"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
	"time"
	"xingjia-management-gateway/apis"
	"xingjia-management-gateway/consts"
	"xingjia-management-gateway/errors"
	"xingjia-management-gateway/impl/v1/operation_log"
	dbmodel "xingjia-management-gateway/model"
)

func checkUserResetPasswordParam(req *apis.UserResetPasswordRequest) error {
	switch {
	case req.Password == "":
		return status.Error(10003, "未输入目标")
	case req.SelfId == 0:
		return status.Error(10003, "未知操作者")
	case req.ResetId == 0:
		return status.Error(10003, "目标账户不能为空")
	}
	return nil
}

//
func UserResetPassword(ctx context.Context, req *apis.UserResetPasswordRequest) (reply *apis.UserResetPasswordReply, err error) {

	reply = &apis.UserResetPasswordReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	if err = checkUserResetPasswordParam(req); err != nil {
		return nil, err
	}
	if err = checkAdmin(req.SelfId); err != nil {
		return nil, err
	}

	old := &dbmodel.TUser{}
	where := map[string]interface{}{
		"id": req.ResetId,
	}
	err = old.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if old.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}

	values := map[string]interface{}{}
	now := time.Now()

	passwd, _ := util.GetMd5Pass(req.Password, consts.CRYPTO_KEY)
	values["password"] = passwd
	values["updated_at"] = now
	p := dbmodel.TUser{}

	db := database.DB().Begin()
	defer func() {
		if err != nil {
			db.Rollback()
		}
	}()

	err = p.Update(db, where, values)
	if err != nil {
		return nil, errors.DataBaseError
	}

	// 操作日志
	content := getUserContent(*old)
	logReq := &apis.OperationLogAddRequest{
		OriginContent: "",
		TargetContent: content,
		Uid:           req.SelfId,
		User:          req.SelfName,
		Module:        consts.OperationModuleAccount,
		OperationTime: time.Now().Unix(),
		Operation:     consts.OperationActionUserResetPassword,
	}
	err = operation_log.OperationLogAddHandle(logReq, db)
	if err != nil {
		return nil, err
	}
	db.Commit()

	return reply, nil
}
